import { useReducer, useMemo } from "react";
import createUseContext from "constate";
import { tryCreateDevToolsLogger } from "./devtools";
export function useMethods(initialState, methods) {
    // 1. Create actions and action creators from map of actions
    const reducer = useMemo(() => {
        const logger = tryCreateDevToolsLogger();
        return (state, action) => {
            const nextState = methods[action.type](state)(...action.payload);
            if (logger) {
                logger(action, nextState);
            }
            return nextState;
        };
    }, []);
    const [state, dispatch] = useReducer(reducer, initialState);
    const methodsOut = {};
    Object.keys(methods).forEach(type => {
        const originalMethodIn = methods[type];
        const methodOut = (...payload) => {
            dispatch({
                type: type,
                payload
            });
        };
        // @ts-ignore check it later
        methodsOut[type] = methodOut;
    });
    return { state, methods: methodsOut };
}
export const createUseStore = (defaultState, methodsIn) => createUseContext(({ initialState }) => useMethods(initialState || defaultState, methodsIn), ({ state }) => [state]);
